"""Argparser module for pinging"""
from jina.parsers.base import set_base_parser


def set_ping_parser(parser=None):
    """Set the parser for `ping`

    :param parser: an existing parser to build upon
    :return: the parser
    """
    if not parser:
        parser = set_base_parser()

    parser.add_argument(
        'host', type=str, help='The host address of the target Pod, e.g. 0.0.0.0'
    )
    parser.add_argument(
        'port', type=int, help='The control port of the target deployment/pod'
    )
    parser.add_argument(
        '--timeout',
        type=int,
        default=3000,
        help='''
Timeout in millisecond of one check
-1 for waiting forever
''',
    )
    parser.add_argument(
        '--retries',
        type=int,
        default=3,
        help='The max number of tried health checks before exit with exit code 1',
    )
    return parser



def get_full_schema() -> dict:
    """Get full schema
    :return: the full schema for Jina core as a dict.
    """
    from jina import __version__
    from jina.importer import IMPORTED
    from jina.schemas.executor import schema_all_executors
    from jina.schemas.flow import schema_flow
    from jina.schemas.meta import schema_metas
    from jina.schemas.deployment import schema_deployment

    definitions = {}
    for s in [
        schema_all_executors,
        schema_flow,
        schema_metas,
        schema_deployment,
        IMPORTED.schema_executors,
    ]:
        definitions.update(s)

    return {
        '$id': f'https://api.jina.ai/schemas/{__version__}.json',
        '$schema': 'http://json-schema.org/draft-07/schema#',
        'description': 'The YAML schema of Jina objects (Flow, Executor).',
        'type': 'object',
        'oneOf': [{'$ref': '#/definitions/Jina::Flow'}]
        + [{"$ref": f"#/definitions/{k}"} for k in IMPORTED.schema_executors.keys()],
        'definitions': definitions,
    }



from jina.resources.health_check.pod import check_health_pod


def check_health_http(addr):
    import requests

    try:
        resp = requests.get(f'http://{addr}/')
        if not resp.status_code == 200:
            raise RuntimeError(
                f'The http gateway is unhealthy http status : {resp.status_code}'
            )
    except requests.exceptions.RequestException as e:
        print('The http gateway is unhealthy')
        raise e

    print('The http gateway is healthy')


async def check_health_websocket(addr):
    import websockets

    try:
        async with websockets.connect(f'ws://{addr}') as websocket:
            pass
    except websockets.exceptions.WebSocketException as e:
        print('The websocket gateway is unhealthy')
        raise e

    print('The websocket gateway is healthy')


if __name__ == '__main__':
    """
    Health check cli (for docker):

    Example:
        python jina.resources.health_check.pod localhost:1234 http
    """
    import sys

    if len(sys.argv) < 3:
        raise ValueError(
            'You need to specify a address to check health and at protocol'
        )

    addr = sys.argv[1]
    protocol = sys.argv[2]

    if protocol == 'grpc':
        check_health_pod(addr)
    elif protocol == 'http':
        check_health_http(addr)
    elif protocol == 'websocket':
        import asyncio

        asyncio.run(check_health_websocket(addr))
    else:
        raise ValueError(f'{protocol} should be in ["grpc","http","websocket"]')



"""Argparser module for Pod runtimes"""
import argparse

from jina import helper
from jina.enums import PodRoleType
from jina.parsers.helper import _SHOW_ALL_ARGS, KVAppendAction, add_arg_group


def mixin_pod_parser(parser, port_monitoring=True):
    """Mixing in arguments required by :class:`Pod` into the given parser.
    :param parser: the parser instance to which we add arguments
    :param port_monitoring: if to include the port parsing
    """

    gp = add_arg_group(parser, title='Pod')

    gp.add_argument(
        '--runtime-cls',
        type=str,
        default='WorkerRuntime',
        help='The runtime class to run inside the Pod',
    )

    gp.add_argument(
        '--timeout-ready',
        type=int,
        default=600000,
        help='The timeout in milliseconds of a Pod waits for the runtime to be ready, -1 for waiting '
        'forever',
    )

    gp.add_argument(
        '--env',
        action=KVAppendAction,
        metavar='KEY: VALUE',
        nargs='*',
        help='The map of environment variables that are available inside runtime',
    )

    # hidden CLI used for internal only

    gp.add_argument(
        '--shard-id',
        type=int,
        default=0,
        help='defines the shard identifier for the executor. It is used as suffix for the workspace path of the executor`'
        if _SHOW_ALL_ARGS
        else argparse.SUPPRESS,
    )

    gp.add_argument(
        '--pod-role',
        type=PodRoleType.from_string,
        choices=list(PodRoleType),
        default=PodRoleType.WORKER,
        help='The role of this Pod in a Deployment'
        if _SHOW_ALL_ARGS
        else argparse.SUPPRESS,
    )

    gp.add_argument(
        '--noblock-on-start',
        action='store_true',
        default=False,
        help='If set, starting a Pod/Deployment does not block the thread/process. It then relies on '
        '`wait_start_success` at outer function for the postpone check.'
        if _SHOW_ALL_ARGS
        else argparse.SUPPRESS,
    )

    gp.add_argument(
        '--shards',
        type=int,
        default=1,
        help='The number of shards in the deployment running at the same time. For more details check '
        'https://docs.jina.ai/fundamentals/flow/create-flow/#complex-flow-topologies',
    )

    gp.add_argument(
        '--replicas',
        type=int,
        default=1,
        help='The number of replicas in the deployment',
    )

    gp.add_argument(
        '--port',
        type=int,
        default=helper.random_port(),
        help='The port for input data to bind to, default is a random port between [49152, 65535]',
    )

    gp.add_argument(
        '--monitoring',
        action='store_true',
        default=False,
        help='If set, spawn an http server with a prometheus endpoint to expose metrics',
    )

    if port_monitoring:
        gp.add_argument(
            '--port-monitoring',
            type=int,
            default=helper.random_port(),
            dest='port_monitoring',
            help=f'The port on which the prometheus server is exposed, default is a random port between [49152, 65535]',
        )

    gp.add_argument(
        '--retries',
        type=int,
        default=-1,
        dest='retries',
        help=f'Number of retries per gRPC call. If <0 it defaults to max(3, num_replicas)',
    )

    gp.add_argument(
        '--floating',
        action='store_true',
        default=False,
        help='If set, the current Pod/Deployment can not be further chained, '
        'and the next `.add()` will chain after the last Pod/Deployment not this current one.',
    )
